%%%% Structural changes in the US economy: is there a role for monetary policy? 
%%%% Canova Fabio and Luca Gambetti
%%%% Replication file

%%%% estimate the model
TvcAllVarKSCCanovaGambetti2008bis
save ResultsCanovaGambetti2008

%%%% identify the shock
StrAnalysisSignCanovaGambetti2008
save ResultsCanovaGambetti2008

%%%% standard deviation of the shock
for i=1:800
    mpstd(:,i)=1./dimp1(i,:)';
end
smp=sort(mpstd,2);

%%%% standardized impulse response functions
for j=1:4
    for k=1:12,
        irfStd(j,k,:,:)=squeeze(irf1(j,1,k,:,:))./squeeze(mpstd(:,:))';
    end
end
cirf=squeeze(sum(irfStd.^2,2));
scirf=sort(cirf,2);

%%%% compute Taylor rule long run coefficients
idx=1:4:160-1;
ind1=squeeze(ind1);
for i=1:800
    for t=1:40
        gam1(:,t,i)=kron(ra1(:,1,i,t)'*inv(chol(PR(:,:,idx(t),ind1(i,t))))',eye(9))*PB(:,idx(t),ind1(i,t));
        gam0(:,t,i)=ra1(:,1,i,t)'*inv(chol(PR(:,:,idx(t),ind1(i,t))))';
        gam2(:,t,i)=gam0(:,t,i)./gam0(1,t,i);
        gam3(:,t,i)=gam1(:,t,i)./gam0(1,t,i);
        tay1(t,i)=(-gam0(2,t,i)+gam1(3,t,i)+gam1(7,t,i))/(gam0(1,t,i)-gam1(2,t,i)-gam1(6,t,i));
        tay2(t,i)=(-gam0(3,t,i)+gam1(4,t,i)+gam1(8,t,i))/(gam0(1,t,i)-gam1(2,t,i)-gam1(6,t,i));
        tay3(t,i)=(-gam0(4,t,i)+gam1(5,t,i)+gam1(9,t,i))/(gam0(1,t,i)-gam1(2,t,i)-gam1(6,t,i));
        tay(t,i)=(-gam0(1,t,i)+gam1(2,t,i)+gam1(6,t,i))/(gam0(4,t,i)-gam1(5,t,i)-gam1(9,t,i));
    end
end
sgam1=sort(gam1,3);
sgam0=sort(gam0,3);
sgam3=sort(gam3,3);
sgam2=sort(gam2,3);
stay1= sort(tay1,2);
stay2= sort(tay2,2);
stay3= sort(tay3,2);


%%%% compute unconditional variance and persistence
[v1,v2,va,p1,p2,pe]=TvcGetVarianceCanovaGambetti2008(PB,PR,irf1,ind1,ra1,2,4,1,4,1);
save ResultsCanovaGambetti2008
pestd=pe(1:n,1:n,:,:)./va; % standardized persistence
spestd=sort(pestd,3);
sva=sort(va,3);

%%%% explained variance and persistence
vaex=squeeze(v2(:,:,:,:))./squeeze(sum(v1(:,:,:,:),2));
svaex=sort(vaex,2);
paex=squeeze(p2(:,:,:,:))./squeeze(sum(p1(:,:,:,:),2));
spaex=sort(paex,2);


%%%% compute responses to coefficients shocks
% tt=0;
% for t=1:4:52%size(PB,2)
%     tt=tt+1
%     [my1(:,:,:,tt),my2(:,:,:,tt),coeff1(:,:,tt),coeff2(:,:,tt)]=TvcSystIrf(PB(:,t,ind1(:,tt)),PQ(:,:,ind1(:,tt)), ...
%         squeeze(PR(:,:,t,ind1(:,tt))),ra1(:,:,:,tt),x(t,:)',2,4,1,40,[3 7],20);
% end

tt=0;
for t=1:4:52%size(PB,2)
    tt=tt+1
    [my1b(:,:,:,tt),my2b(:,:,:,tt),coeff1b(:,:,tt),coeff2b(:,:,tt)]=TvcSystIrf(PB(:,t,ind1(:,tt)),PQ(:,:,ind1(:,tt)), ...
        squeeze(PR(:,:,t,ind1(:,tt))),ra1(:,:,:,tt),x(t,:)',2,4,1,40,[2 3 6 7],50);
end
save ResultsCanovaGambetti2008

%%%% irf of persistence 
tt=0;
for t=1:4:4*13 
    tt=tt+1
    [m1,m2,va1(:,:,:,tt),va2(:,:,:,tt),pe1(:,:,:,tt),pe2(:,:,:,tt)]=TvcSpecDensIrf(coeff1(:,:,tt),coeff2(:,:,tt),...
        squeeze(PR(:,:,t,ind1(:,tt))),2,4,1,1);
end

%%%% sacrifice ratio
% densr=squeeze(mean(my1(2,:,:,1:13)-my2(2,:,:,1:13),3));
% numsr=squeeze(mean(my1(3,:,:,1:13)-my2(3,:,:,1:13),3));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIGURES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% time
tim=(1967:2006); % dates

%%%% confidence levels for the bands
lb=floor(size(irf1,4)*.16)+1;  % lower band
ub=ceil(size(irf1,4)*.84)+1;   % upper band

ssva=sqrt(sva);

%%%% Figure 1
% figure(1)
% subplot(2,2,1),plot(tim,squeeze(median(sqrt(sva(2,2,:,:)),3)),'k', tim,squeeze(sqrt(sva(2,2,[lb ub],:)))',':k'),...
%     axis tight,title('Inflation'),ylabel('Standard Deviation')
% subplot(2,2,2),plot(tim,squeeze(median(sqrt(sva(3,3,:,:)),3)),'k', tim,squeeze(sqrt(sva(3,3,[lb ub],:)))',':k'),...
%     axis tight,title('Output growth')
% subplot(2,2,3),plot(tim,squeeze(median(pestd(2,2,:,:),3)),'k', tim,squeeze(spestd(2,2,[lb ub],:))',':k'),...
%     axis tight,ylabel('Persistence')
% subplot(2,2,4),plot(tim,squeeze(median(pestd(3,3,:,:),3)),'k', tim,squeeze(spestd(3,3,[lb ub],:))',':k'),axis tight

figure(1)
subplot(2,2,1),plot(tim,squeeze(median(ssva(2,2,:,:),3)),'k', tim,squeeze(ssva(2,2,[lb ub],:))',':k'),...
    axis tight,title('Inflation'),ylabel('Standard Deviation')
subplot(2,2,2),plot(tim,squeeze(median(ssva(3,3,:,:),3)),'k', tim,squeeze(ssva(3,3,[lb ub],:))',':k'),...
    axis tight,title('Output growth')
subplot(2,2,3),plot(tim,squeeze(median(pestd(2,2,:,:),3)),'k', tim,squeeze(spestd(2,2,[lb ub],:))',':k'),...
    axis tight,ylabel('Persistence')
subplot(2,2,4),plot(tim,squeeze(median(pestd(3,3,:,:),3)),'k', tim,squeeze(spestd(3,3,[lb ub],:))',':k'),axis tight

%%%% Figure 2a 2b
figure(2)
[mx my]=meshgrid(tim,1:12);
surf(mx,my,squeeze(median(irfStd(2,:,:,:),3))),axis tight
figure(3)
surf(mx,my,squeeze(median(irfStd(3,:,:,:),3))),axis tight

%%%% Figure 2c 2d
figure(4)
subplot(2,1,1),plot(tim,squeeze(median(cirf(2,:,:),2)),'k',tim,squeeze(scirf(2,[lb ub],:))',':k'),axis tight,title('Inflation')
subplot(2,1,2),plot(tim,squeeze(median(cirf(3,:,:),2)),'k',tim,squeeze(scirf(3,[lb ub],:))',':k'),axis tight,title('Output growth')

%%%% Figure 3
figure(6)
plot(tim,squeeze(median(mpstd,2)),'k',tim, smp(:,[lb ub]),':k'),axis tight

%%%% Figure 4
tim=(1967:2006);
figure(7)
subplot(2,2,1),plot(tim,squeeze(median(vaex(2,:,:),2)),'k', tim,squeeze(svaex(2,[lb ub],:))',':k'),...
    axis tight,title('Inflation'),ylabel('Variance')
subplot(2,2,2),plot(tim,squeeze(median(vaex(3,:,:),2)),'k', tim,squeeze(svaex(3,[lb ub],:))',':k'),...
    axis tight,title('Output growth')
subplot(2,2,3),plot(tim,squeeze(median(paex(3,:,:),2)),'k', tim,squeeze(spaex(3,[lb ub],:))',':k'),...
    axis tight,ylabel('Persistence')
subplot(2,2,4),plot(tim,squeeze(median(paex(2,:,:),2)),'k', tim,squeeze(spaex(2,[lb ub],:))',':k'),axis tight

%%%% Figure 5
figure(8)
subplot(3,1,1),plot(tim,median(tay1,2),'k',tim, stay1(:,[lb ub]),':k',tim, ones(40,1),'--k'),axis tight,title('Inflation')
subplot(3,1,2),plot(tim,median(tay2,2),'k',tim, stay2(:,[lb ub]),':k',tim, ones(40,1),'--k'),axis tight,title('Output growth')
subplot(3,1,3),plot(tim,median(tay3,2),'k',tim, stay3(:,[lb ub]),':k',tim, ones(40,1),'--k'),axis tight,title('Money')

%%%% Figure 6a
[tx ty]=meshgrid(1967:1979,1:40);
figure(9)
surf(tx,ty,squeeze(median(my1(2,:,:,1:13)-my2(2,:,:,1:13),3))),axis tight

%%%% Figure 6b
figure(10)
surf(tx,ty,squeeze(median(my1(3,:,:,1:13)-my2(3,:,:,1:13),3))),axis tight

%%%% Figure 7
figure(11)
plot(1967:1979,squeeze(median(pe1(2,2,:,:)./va1(2,2,:,:),3)),':k',1967:1979,squeeze(median(pe2(2,2,:,:)./va2(2,2,:,:),3)),'k'),axis tight

%%%% Figure 8
figure(12)
plot(1967:1979,sum(numsr(1:2,:))./sum(densr(1:2,:)),'k',1967:1979,sum(numsr(1:4,:))./sum(densr(1:4,:)),'--k'),axis tight